#ifndef MY_ATK_STREAMABLE_CONTENT_H
#define MY_ATK_STREAMABLE_CONTENT_H

/*
 * MyAtkStreamableContent: derives from GObject and implements AtkStreamableContent
 */

#include <atk/atk.h>
#include <glib-object.h>

#define MY_TYPE_ATK_STREAMABLE_CONTENT             (my_atk_streamable_content_get_type ())
#define MY_ATK_STREAMABLE_CONTENT(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), MY_TYPE_ATK_STREAMABLE_CONTENT, MyAtkStreamableContent))
#define MY_ATK_STREAMABLE_CONTENT_CLASS(vtable)    (G_TYPE_CHECK_CLASS_CAST ((vtable), MY_TYPE_ATK_STREAMABLE_CONTENT, MyAtkStreamableContentClass))
#define MY_IS_ATK_STREAMABLE_CONTENT(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MY_TYPE_ATK_STREAMABLE_CONTENT))
#define MY_IS_ATK_STREAMABLE_CONTENT_CLASS(vtable) (G_TYPE_CHECK_CLASS_TYPE ((vtable), MY_TYPE_ATK_STREAMABLE_CONTENT))
#define MY_ATK_STREAMABLE_CONTENT_GET_CLASS(inst)  (G_TYPE_INSTANCE_GET_CLASS ((inst), MY_TYPE_ATK_STREAMABLE_CONTENT, MyAtkStreamableContentClass))

typedef struct _MyAtkStreamableContent MyAtkStreamableContent;
typedef struct _MyAtkStreamableContentClass MyAtkStreamableContentClass;

static const gchar* mime_types[]={"text/plain", "text/richtext"};
static const gchar* file_names[]={"file1", "file2"};
struct _MyAtkStreamableContent
{
    GObject parent;
};

struct _MyAtkStreamableContentClass
{
    GObjectClass parent;
};

GType my_atk_streamable_content_get_type();
//*************************implementation***********************
//implementation of virtual functions
//*****************get_n_mime_types************
static gint my_atk_streamable_content_get_n_mime_types(
    AtkStreamableContent *streamable)
{
    g_return_val_if_fail(MY_IS_ATK_STREAMABLE_CONTENT(streamable), 0);
    
    return sizeof(mime_types) / sizeof(mime_types[0]);
}
//*****************get_mime_type****************
static const gchar* my_atk_streamable_content_get_mime_type(
    AtkStreamableContent *streamable,
    gint i)
{
    g_return_val_if_fail(MY_IS_ATK_STREAMABLE_CONTENT(streamable), NULL);
    
    if((i < 0) || (i >= sizeof(mime_types) / sizeof(mime_types[0])))
    {
        return NULL;
    }
    return mime_types[i];
}
//**********************get_stream*******************
static GIOChannel* my_atk_streamable_content_get_stream(
    AtkStreamableContent *streamable,
    const gchar* mime_type)
{
    g_return_val_if_fail(MY_IS_ATK_STREAMABLE_CONTENT(streamable), NULL);
    
    for(gint i = 0; i < sizeof(mime_types) / sizeof(mime_types[0]); i++)
    {
        if(strcmp(mime_type, mime_types[i]) == 0)
        {
            GError *error = NULL;
            gchar* full_filename = T2C_GET_DATA_PATH(file_names[i]);
            GIOChannel* channel = g_io_channel_new_file(full_filename, "r", &error);
            if(error != NULL)
            {
                TRACE("Cannot open file '%s' for read: %s", full_filename,
                    error->message);
                g_error_free(error);
            }
            g_free(full_filename);
            return channel;
        }
    }
    return NULL;
}
//other functions
static void my_atk_streamable_content_interface_init(gpointer g_iface, gpointer iface_data)
{
    AtkStreamableContentIface *klass = (AtkStreamableContentIface*)g_iface;
    
    klass->get_n_mime_types = my_atk_streamable_content_get_n_mime_types;
    klass->get_mime_type = my_atk_streamable_content_get_mime_type;
    klass->get_stream = my_atk_streamable_content_get_stream;
}

GType my_atk_streamable_content_get_type()
{
    static GType type = 0;
    if(type == 0)
    {
        static const GTypeInfo typeInfo = 
        {
            sizeof(MyAtkStreamableContentClass),
            NULL, //base_init
            NULL, //base_finalize
            NULL, //class_init
            NULL, //class_finalize
            NULL, //class_data
            sizeof(MyAtkStreamableContent),
            0, //n_preallocs
            NULL //instance_init
        };

        static const GInterfaceInfo iface_info = 
        {
            my_atk_streamable_content_interface_init,    /* interface_init*/
            NULL,                               /* interface_finalize*/
            NULL                                /* interface_data */
        };
        type = g_type_register_static(G_TYPE_OBJECT, "MyAtkStreamableContent", &typeInfo, 0);
        g_type_add_interface_static(type,
            ATK_TYPE_STREAMABLE_CONTENT,
            &iface_info);
    }
    return type;    
}
#endif /*MY_ATK_STREAMABLE_CONTENT_H*/
